@@ -110,7 +110,7 @@ class TestAttributeValidation(base.BaseTestCase):
         msg = validators.validate_values("1", ["2", "1", "4", "5"])
         self.assertIsNone(msg)
 
-        # Check that value is not compatible for comparision
+        # Check that value is not compatible for comparison
         response = "'valid_values' does not support membership operations"
         self.assertRaisesRegex(TypeError, response,
                                validators.validate_values, data=None,
@@ -1026,3 +1026,100 @@ class TestAttributeValidation(base.BaseTestCase):
             {'port_id': '11111111-ffff-ffff-ffff-000000000000'},
         ]
         self.assertIsNone(validators.validate_subports(body))
+
+
+class TestValidateIPSubnetNone(base.BaseTestCase):
+
+    def test_validate_none(self):
+        self.assertIsNone(validators.validate_ip_or_subnet_or_none(None))
+
+    def test_validate_ipv4(self):
+        testdata = "172.0.0.1"
+        self.assertIsNone(validators.validate_ip_or_subnet_or_none(testdata))
+
+    def test_validate_ipv4_subnet(self):
+        testdata = "172.0.0.1/24"
+        self.assertIsNone(validators.validate_ip_or_subnet_or_none(testdata))
+
+    def test_validate_ipv6(self):
+        testdata = "2001:0db8:0a0b:12f0:0000:0000:0000:0001"
+        self.assertIsNone(validators.validate_ip_or_subnet_or_none(testdata))
+
+    def test_validate_ipv6_subnet(self):
+        testdata = "::1/128"
+        self.assertIsNone(validators.validate_ip_or_subnet_or_none(testdata))
+
+    def test_validate_ipv4_invalid(self):
+        testdata = "300.0.0.1"
+        self.assertEqual(("'300.0.0.1' is neither a valid IP address, nor is "
+                          "it a valid IP subnet"),
+                         validators.validate_ip_or_subnet_or_none(testdata))
+
+    def test_validate_ipv4_subnet_invalid(self):
+        testdata = "172.0.0.1/45"
+        self.assertEqual(("'172.0.0.1/45' is neither a valid IP address, nor "
+                          "is it a valid IP subnet"),
+                         validators.validate_ip_or_subnet_or_none(testdata))
+
+    def test_validate_ipv6_invalid(self):
+        testdata = "xxxx:0db8:0a0b:12f0:0000:0000:0000:0001"
+        self.assertEqual(("'xxxx:0db8:0a0b:12f0:0000:0000:0000:0001' is "
+                          "neither a valid IP address, nor is it a valid IP "
+                          "subnet"),
+                         validators.validate_ip_or_subnet_or_none(testdata))
+
+    def test_validate_ipv6_subnet_invalid(self):
+        testdata = "::1/2048"
+        self.assertEqual(("'::1/2048' is neither a valid IP address, nor is "
+                          "it a valid IP subnet"),
+                         validators.validate_ip_or_subnet_or_none(testdata))
+
+
+class TestPortRangeValidation(base.BaseTestCase):
+
+    def test_valid_port(self):
+        result = validators.validate_port_range_or_none("80")
+        self.assertIsNone(result)
+
+    def test_valid_range(self):
+        result = validators.validate_port_range_or_none("80:8888")
+        self.assertIsNone(result)
+
+    def test_port_too_high(self):
+        result = validators.validate_port_range_or_none("99999")
+        self.assertEqual(u"Invalid port: 99999.", result)
+
+    def test_port_too_low(self):
+        result = validators.validate_port_range_or_none("-1")
+        self.assertEqual(u"Invalid port: -1.", result)
+
+    def test_range_too_high(self):
+        result = validators.validate_port_range_or_none("80:99999")
+        self.assertEqual(u"Invalid port: 99999.", result)
+
+    def test_range_too_low(self):
+        result = validators.validate_port_range_or_none("-1:8888")
+        self.assertEqual(u"Invalid port: -1.", result)
+
+    def test_range_wrong_way(self):
+        result = validators.validate_port_range_or_none("8888:80")
+        self.assertEqual(u"First port in a port range must be lower than the "
+                         "second port.", result)
+
+    def test_range_invalid(self):
+        result = validators.validate_port_range_or_none("DEAD:BEEF")
+        self.assertEqual(u"Invalid port: DEAD.", result)
+
+    def test_range_bad_input(self):
+        result = validators.validate_port_range_or_none(['a', 'b', 'c'])
+        self.assertEqual(u"Port range must be a string.", result)
+
+    def test_range_colon(self):
+        result = validators.validate_port_range_or_none(":")
+        self.assertEqual(u"Port range must be two integers separated by a "
+                         "colon.", result)
+
+    def test_too_many_colons(self):
+        result = validators.validate_port_range_or_none("80:888:8888")
+        self.assertEqual(u"Port range must be two integers separated by a "
+                         "colon.", result)
